Imports System.ComponentModel
Imports System.ServiceProcess
Imports System.Text
Imports System.Threading.Tasks
Imports System.IO
Imports System.Configuration
Imports System.Xml.Serialization

Namespace VB_FileSystemWatcher
    ''' <summary>
    ''' This class will run as a Windows Service intended to monitor what files have been added to the filesystem and triggering some actions as a response
    ''' </summary>
    Partial Public Class Service1
        Inherits ServiceBase

#Region "Private variables"
        ''' <summary>
        ''' Keeps a list of all the file watcher listeners, predefined on an XML file
        ''' </summary>
        Private listFolders As List(Of CustomFolderSettings)

        ''' <summary>
        ''' Keeps a list of all the file system watchers in execution.
        ''' </summary>
        Private listFileSystemWatcher As List(Of FileSystemWatcher)

        ''' <summary>
        ''' Name of the XML file where resides the specification of folders and extensions to be monitored
        ''' </summary>
        Private fileNameXML As String
#End Region

#Region "Constructors"
        ''' <summary>
        ''' Default Constructor
        ''' </summary>
        Public Sub New()
            InitializeComponent()
        End Sub
#End Region

#Region "Protected Methods"
        ''' <summary>
        ''' Event automatically fired when the service is started by Windows
        ''' </summary>
        ''' <param name="args">array of arguments</param>
        Protected Overrides Sub OnStart(ByVal args() As String)
            ' Initialize the list of filesystem Watchers based on the XML configuration file
            PopulateListFileSystemWatchers()

            ' Start the file system watcher for each of the file specification and folders found on the List<>
            StartFileSystemWatcher()
        End Sub

        ''' <summary>
        ''' Event automatically fired when the service is stopped by Windows
        ''' </summary>
        Protected Overrides Sub OnStop()
            If listFileSystemWatcher IsNot Nothing Then
                For Each fsw As FileSystemWatcher In listFileSystemWatcher
                    ' Stop listening
                    fsw.EnableRaisingEvents = False

                    ' Record a log entry into Windows Event Log
                    CustomLogEvent(String.Format("Stop monitoring files with extension ({0}) in the folder ({1})", fsw.Filter, fsw.Path))

                    ' Dispose the Object
                    fsw.Dispose()
                Next fsw

                ' Cleans the list
                listFileSystemWatcher.Clear()
            End If
        End Sub
#End Region

#Region "Private Methods"
        ''' <summary>
        ''' Stops the main execution of the Windows Service
        ''' </summary>
        Private Sub StopMainExecution()
            If listFileSystemWatcher IsNot Nothing Then
                For Each fsw As FileSystemWatcher In listFileSystemWatcher
                    ' Stop listening
                    fsw.EnableRaisingEvents = False

                    ' Record a log entry into Windows Event Log
                    CustomLogEvent(String.Format("Stop monitoring files with extension ({0}) in the folder ({1})", fsw.Filter, fsw.Path))

                    ' Dispose the Object
                    fsw.Dispose()
                Next fsw

                ' Cleans the list
                listFileSystemWatcher.Clear()
            End If
        End Sub

        ''' <summary>
        ''' Start the file system watcher for each of the file specification and folders found on the List<>
        ''' </summary>
        Private Sub StartFileSystemWatcher()
            ' Creates a new instance of the list
            Me.listFileSystemWatcher = New List(Of FileSystemWatcher)

            ' Loop the list to process each of the folder specifications found
            For Each customFolder As CustomFolderSettings In listFolders
                Dim dir As New DirectoryInfo(customFolder.FolderPath)

                ' Checks whether the SystemWatcher is enabled and also the directory is a valid location
                If customFolder.FolderEnabled AndAlso dir.Exists Then
                    ' Creates a new instance of FileSystemWatcher
                    Dim fileSWatch As New FileSystemWatcher

                    ' Sets the filter
                    fileSWatch.Filter = customFolder.FolderFilter

                    ' Sets the folder location
                    fileSWatch.Path = customFolder.FolderPath

                    ' Sets the action to be executed
                    Dim actionToExecute As New StringBuilder(customFolder.ExecutableFile)

                    ' List of arguments
                    Dim actionArguments As New StringBuilder(customFolder.ExecutableArguments)

                    ' Subscribe to notify filters
                    fileSWatch.NotifyFilter = NotifyFilters.LastWrite Or NotifyFilters.FileName Or NotifyFilters.DirectoryName

                    ' Associate the event that will be triggered when a new file is added to the monitored folder, using a lambda Expression                    
                    AddHandler fileSWatch.Created, Sub(senderObj, fileSysArgs) fileSWatch_Created(senderObj, fileSysArgs, actionToExecute.ToString, actionArguments.ToString)

                    ' Begin watching
                    fileSWatch.EnableRaisingEvents = True

                    ' Add the systemWatcher to the list
                    listFileSystemWatcher.Add(fileSWatch)

                    ' Record a log entry into Windows Event Log
                    CustomLogEvent(String.Format("Starting to monitor files with extension ({0}) in the folder ({1})", fileSWatch.Filter, fileSWatch.Path))
                Else
                    ' Record a log entry into Windows Event Log
                    CustomLogEvent(String.Format("File system monitor cannot start because the folder ({0}) does not exist", customFolder.FolderPath))
                End If
            Next customFolder
        End Sub

        ''' <summary>
        ''' This event is triggered when a file with the specified extension is created on the monitored folder
        ''' </summary>
        ''' <param name="sender">Object raising the event</param>
        ''' <param name="e">List of arguments - FileSystemEventArgs</param>
        ''' <param name="action_Exec">The action_ execute.</param>
        ''' <param name="action_Args">The action_ arguments.</param>
        Private Sub fileSWatch_Created(ByVal sender As Object, ByVal e As FileSystemEventArgs, ByVal action_Exec As String, ByVal action_Args As String)
            ' Gets the name of the file recently added
            Dim fileName As String = e.FullPath

            ' Adds the file name to the arguments.  The filename will be placed in lieu of {0}
            Dim newStr As String = String.Format(action_Args, fileName)

            ' Executes the command from the DOS window
            ExecuteCommandLineProcess(action_Exec, newStr)
        End Sub

        ''' <summary>
        ''' Record messages and logs into the Windows Event log
        ''' </summary>
        ''' <param name="message">Message to be recorded into the Windows Event log</param>
        Private Sub CustomLogEvent(ByVal message As String)
            Dim eventSource As String = Me.ServiceName
            Dim dt As New Date
            dt = Date.UtcNow
            message = dt.ToLocalTime & ": " & message

            EventLog.WriteEntry(eventSource, message)
        End Sub

        ''' <summary>
        ''' Reads an XML file and populates a list of <CustomFolderSettings>
        ''' </summary>
        Private Sub PopulateListFileSystemWatchers()
            ''' Get the XML file name from the APP.Config file
            fileNameXML = ConfigurationManager.AppSettings("XMLFileFolderSettings")

            ' Creates an instance of XMLSerializer
            Dim deserializer As New XmlSerializer(GetType(List(Of CustomFolderSettings)))

            Dim reader As TextReader = New StreamReader(fileNameXML)
            Dim obj As Object = deserializer.Deserialize(reader)

            ' Close the TextReader object
            reader.Close()

            ' Obtains a list of strongly typed CustomFolderSettings from XML Input data
            listFolders = TryCast(obj, List(Of CustomFolderSettings))
        End Sub

        ''' <summary>
        ''' Executes a set of instructions through the command window
        ''' </summary>
        ''' <param name="executableFile">Name of the executable file or program</param>
        ''' <param name="argumentList">List of arguments</param>
        Private Sub ExecuteCommandLineProcess(ByVal executableFile As String, ByVal argumentList As String)
            ' Use ProcessStartInfo class.
            Dim startInfo As New ProcessStartInfo
            startInfo.CreateNoWindow = True
            startInfo.UseShellExecute = False
            startInfo.FileName = executableFile
            startInfo.WindowStyle = ProcessWindowStyle.Hidden
            startInfo.Arguments = argumentList

            Try
                ' Start the process with the info we specified.
                ' Call WaitForExit and then the using-statement will close.
                Using exeProcess As Process = Process.Start(startInfo)
                    exeProcess.WaitForExit()

                    ' Register a log of the successful operation
                    CustomLogEvent(String.Format("Succesful operation --> Executable: {0} --> Arguments: {1}", executableFile, argumentList))
                End Using
            Catch exc As Exception
                ' Register a Log of the Exception
                CustomLogEvent(exc.Message)
            End Try
        End Sub
#End Region

    End Class
End Namespace
